from pytidy.registry import Registry
from typing import Type, Dict


_reg: Registry = Registry.get_instance()


def autowired(func):
    # @functools.wraps
    def wrapped(*args, **kwargs):
        arg_defs: Dict[str, Type] = func.__annotations__
        for arg_name, klass in arg_defs.items():
            kwargs[arg_name] = _reg.get(klass)
        return func(*args, **kwargs)
    return wrapped


def component(cls):
    _reg.register(cls)
    return cls



# from __future__ import annotations  # TODO: Limited to python3.7
from typing import ClassVar, Dict, Type, TypeVar
from pytidy.error import RegistrationError, RetrievingError

from logging import getLogger
logger = getLogger(__file__)


T = TypeVar('T')


class Registry:
    __instance: "Registry" = None
    klasses: ClassVar[Dict[str, object]] = {}

    @staticmethod
    def get_instance() -> "Registry":
        if Registry.__instance is None:
            Registry.__instance = Registry()
        return Registry.__instance

    def __init__(self):
        # TODO: Prevent users from direct invoke
        pass

    def register(self, klass: Type):
        name: str = self.__get_name(klass)
        if name in self.klasses:
            raise RegistrationError("Already registered class: {}".format(name))
        obj = klass()
        self.klasses[name] = obj
        logger.debug("{} is successfully registered to registry".format(name))

    def get(self, klass: Type[T]) -> T:
        name: str = self.__get_name(klass)
        if name not in self.klasses:
            raise RetrievingError("{} is not registered".format(name))

        obj: object = self.klasses[name]
        if not isinstance(obj, klass):
            raise RetrievingError("Registered object is not compatible with {}".format(name))
        return obj

    def __get_name(self, klass: Type) -> str:
        return ".".join((klass.__module__, klass.__name__))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pytidy",
    version="0.1.0",
    author="Takuro Wada",
    author_email="taxpon@gmail.com",
    description="Dependency Injection library based on type hints",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/taxpon/pytidy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
)